<?php

namespace Drupal\symfony_debug\EventSubscriber;

use Symfony\Component\Debug\Debug;
use Symfony\Component\Debug\Exception\FlattenException;
use Symfony\Component\Debug\ExceptionHandler;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Event\GetResponseForExceptionEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Class KernelEventSubscriber.
 *
 * @package Drupal\symfony_debug\EventSubscriber
 */
class KernelEventSubscriber implements EventSubscriberInterface {

  /**
   * KernelEvents::EXCEPTION event handler.
   *
   * @param \Symfony\Component\HttpKernel\Event\GetResponseForExceptionEvent $event
   *   Event.
   *
   * @throws \Exception
   */
  public function onException(GetResponseForExceptionEvent $event) {
    $exception = $event->getException();
    if (!$exception instanceof FlattenException) {
      $exception = FlattenException::create($exception);
    }

    $handler = new ExceptionHandler();

    // Return a response before the default exception handler can do it.
    $event->setResponse(
      new Response($handler->getHtml($exception), $exception->getStatusCode())
    );
  }

  /**
   * KernelEvents::REQUEST event handler.
   */
  public function onRequest() {
    // This needs to be called as early as possible in order to catch errors.
    // phpcs:ignore
    Debug::enable();
  }

  /**
   * Get subscribed events.
   *
   * @return array
   *   Subscribed events
   */
  public static function getSubscribedEvents() {
    return [
      // Just after ExceptionListener.
      KernelEvents::EXCEPTION => ['onException', -129],
      KernelEvents::REQUEST => 'onRequest',
    ];
  }

}
